@@ -1,14 +1,16 @@
 #!/usr/bin/env python3
-
 import argparse
+import asyncio
 import copy
 import io
 import json
 import os
 import sys
+from asyncio import AbstractEventLoop
 from datetime import datetime
 from decimal import Decimal
 from tempfile import mkstemp
+from threading import Thread
 
 import pkg_resources
 from c8connector import C8Connector, Sample, ConfigAttributeType, Schema
@@ -24,6 +26,8 @@ LOGGER = get_logger('c8_target_postgres')
 DEFAULT_BATCH_SIZE_ROWS = 100000
 DEFAULT_PARALLELISM = 0  # 0 The number of threads used to flush tables
 DEFAULT_MAX_PARALLELISM = 16  # Don't use more than this number of threads by default when flushing streams in parallel
+DEFAULT_BATCH_FLUSH_INTERVAL = 60
+DEFAULT_MIN_BATCH_FLUSH_TIME_GAP = 120
 
 
 class PostgresTargetConnector(C8Connector):
@@ -84,25 +88,31 @@ class PostgresTargetConnector(C8Connector):
             ConfigProperty('batch_size_rows', ConfigAttributeType.INT, False, False,
                            description='Maximum number of rows in each batch. At the end of each batch, '
                                        'the rows in the batch are loaded into Postgres.',
-                           example='(Default: 100000)'),
+                           example='100000'),
+            ConfigProperty('batch_flush_interval', ConfigAttributeType.INT, False, False,
+                           description='Time interval between batch flush task executions in seconds.',
+                           example='60'),
+            ConfigProperty('batch_flush_min_time_gap', ConfigAttributeType.INT, False, False,
+                           description='Minimum time gap between two batch flush tasks in seconds.',
+                           example='120'),
             ConfigProperty('flush_all_streams', ConfigAttributeType.BOOLEAN, False, False,
                            description='Flush and load every stream into Postgres when one batch is full. '
                                        'Warning: This may trigger the COPY command to use files with low '
                                        'number of records.',
-                           example='(Default: False)'),
+                           example='False'),
             ConfigProperty('parallelism', ConfigAttributeType.INT, False, False,
                            description='The number of threads used to flush tables. 0 will create a thread '
                                        'for each stream, up to parallelism_max. -1 will create a thread for '
                                        'each CPU core. Any other positive number will create that number of '
                                        'threads, up to parallelism_max.',
-                           example='(Default: 0)'),
+                           example='0'),
             ConfigProperty('max_parallelism', ConfigAttributeType.INT, False, False,
                            description='Max number of parallel threads to use when flushing tables.',
-                           example='(Default: 16)'),
+                           example='16'),
             ConfigProperty('default_target_schema', ConfigAttributeType.STRING, False, False,
                            description='Name of the schema where the tables will be created. If `schema_mapping` '
                                        'is not defined then every stream sent by the tap is loaded into this schema.',
-                           example='postgres'),
+                           example='public'),
             ConfigProperty('default_target_schema_select_permission', ConfigAttributeType.STRING, False, False,
                            description='Grant USAGE privilege on newly created schemas and grant SELECT privilege '
                                        'on newly created',
@@ -117,33 +127,30 @@ class PostgresTargetConnector(C8Connector):
                                        'Enabling metadata columns will flag the deleted rows by setting the '
                                        '`_SDC_DELETED_AT` metadata column. Without the `add_metadata_columns` '
                                        'option the deleted rows from singer taps will not be recognisable in Postgres.',
-                           example='(Default: False)'),
+                           example='False'),
             ConfigProperty('hard_delete', ConfigAttributeType.BOOLEAN, False, False,
                            description='When `hard_delete` option is true then DELETE SQL commands will be performed '
                                        'in Postgres to delete rows in tables. It is achieved by continuously checking '
                                        'the  `_SDC_DELETED_AT` metadata column sent by the singer tap. Due to deleting '
                                        'rows requires metadata columns, `hard_delete` option automatically enables the '
                                        '`add_metadata_columns` option as well.',
-                           example='(Default: False)'),
+                           example='False'),
             ConfigProperty('data_flattening_max_level', ConfigAttributeType.INT, False, False,
                            description='Object type RECORD items from taps can be transformed to flattened columns by '
                                        'creating columns automatically.<br><br>When value is 0 (default) then '
                                        'flattening functionality is turned off.',
-                           example='(Default: 0)'),
+                           example='0'),
             ConfigProperty('primary_key_required', ConfigAttributeType.BOOLEAN, False, False,
                            description='Log based and Incremental replications on tables with no Primary Key cause '
                                        'duplicates when merging UPDATE events. When set to true, stop loading data '
                                        'if no Primary Key is defined.',
-                           example='(Default: True)'),
+                           example='True'),
             ConfigProperty('validate_records', ConfigAttributeType.BOOLEAN, False, False,
                            description='Validate every single record message to the corresponding JSON schema. '
                                        'This option is disabled by default and invalid RECORD messages will fail '
                                        'only at load time by Postgres. Enabling this option will detect invalid '
                                        'records earlier but could cause performance degradation.',
-                           example='(Default: False)'),
-            ConfigProperty('temp_dir', ConfigAttributeType.STRING, False, False,
-                           description='Directory of temporary CSV files with RECORD messages.',
-                           example='(Default: platform-dependent)'),
+                           example='False'),
         ]
 
     def capabilities(self) -> list[str]:
@@ -224,6 +231,13 @@ def persist_lines(config, lines) -> None:
     stream_to_sync = {}
     total_row_count = {}
     batch_size_rows = config.get('batch_size_rows', DEFAULT_BATCH_SIZE_ROWS)
+    time_schedule = {
+        'interval': config.get('batch_flush_interval', DEFAULT_BATCH_FLUSH_INTERVAL),
+        'last_executed_time': datetime.now(),
+        'min_time_gap': config.get('batch_flush_min_time_gap', DEFAULT_MIN_BATCH_FLUSH_TIME_GAP)
+    }
+    event_loop = setup_flush_task(time_schedule, records_to_load, row_count, stream_to_sync, config,
+                                  state, flushed_state)
 
     # Loop over lines from stdin
     for line in lines:
@@ -295,9 +309,9 @@ def persist_lines(config, lines) -> None:
                                               state,
                                               flushed_state,
                                               filter_streams=filter_streams)
-
                 # emit last encountered state
                 emit_state(copy.deepcopy(flushed_state))
+                time_schedule['last_executed_time'] = datetime.now()
 
         elif t == 'STATE':
             LOGGER.debug('Setting state to %s', o['value'])
@@ -318,9 +332,9 @@ def persist_lines(config, lines) -> None:
             # flush records from previous stream SCHEMA
             if row_count.get(stream, 0) > 0:
                 flushed_state = flush_streams(records_to_load, row_count, stream_to_sync, config, state, flushed_state)
-
                 # emit latest encountered state
                 emit_state(flushed_state)
+                time_schedule['last_executed_time'] = datetime.now()
 
             # key_properties key must be available in the SCHEMA message.
             if 'key_properties' not in o:
@@ -367,9 +381,10 @@ def persist_lines(config, lines) -> None:
     if sum(row_count.values()) > 0:
         # flush all streams one last time, delete records if needed, reset counts and then emit current state
         flushed_state = flush_streams(records_to_load, row_count, stream_to_sync, config, state, flushed_state)
-
+        time_schedule['last_executed_time'] = datetime.now()
     # emit latest state
     emit_state(copy.deepcopy(flushed_state))
+    event_loop.stop()
 
 
 # pylint: disable=too-many-arguments
@@ -380,7 +395,7 @@ def flush_streams(
         config,
         state,
         flushed_state,
-        filter_streams=None):
+        filter_streams=None) -> dict:
     """
     Flushes all buckets and resets records count to 0 as well as empties records to load list
     :param streams: dictionary with records to load per stream
@@ -389,7 +404,7 @@ def flush_streams(
     :param config: dictionary containing the configuration
     :param state: dictionary containing the original state from tap
     :param flushed_state: dictionary containing updated states only when streams got flushed
-    :param filter_streams: Keys of streams to flush from the streams dict. Default is every stream
+    :param filter_streams: Keys of streams to flush from the streams' dict. Default is every stream
     :return: State dict with flushed positions
     """
     parallelism = config.get("parallelism", DEFAULT_PARALLELISM)
@@ -486,6 +501,37 @@ def flush_records(stream, records_to_load, row_count, db_sync, temp_dir=None):
     os.remove(csv_file)
 
 
+def setup_flush_task(time_schedule, streams, row_count, stream_to_sync, config, state, flushed_state,
+                     filter_streams=None) -> AbstractEventLoop:
+    event_loop = asyncio.new_event_loop()
+    Thread(target=start_background_loop, args=(event_loop,), daemon=True).start()
+    asyncio.run_coroutine_threadsafe(
+        flush_task(time_schedule, streams, row_count, stream_to_sync, config, state, flushed_state, filter_streams),
+        event_loop)
+    return event_loop
+
+
+async def flush_task(time_schedule, streams, row_count, stream_to_sync, config, state, flushed_state,
+                     filter_streams=None) -> None:
+    while True:
+        await asyncio.sleep(time_schedule['interval'])
+        timedelta = datetime.now() - time_schedule['last_executed_time']
+        if timedelta.total_seconds() >= time_schedule['min_time_gap']:
+            # if some bucket has records that need to be flushed but haven't reached batch size then flush all buckets.
+            if sum(row_count.values()) > 0:
+                # flush all streams one last time, delete records if needed, reset counts and then emit current state.
+                flushed_state = flush_streams(streams, row_count, stream_to_sync, config, state, flushed_state,
+                                              filter_streams)
+                # emit latest state
+                emit_state(copy.deepcopy(flushed_state))
+                time_schedule['last_executed_time'] = datetime.now()
+
+
+def start_background_loop(loop: asyncio.AbstractEventLoop) -> None:
+    asyncio.set_event_loop(loop)
+    loop.run_forever()
+
+
 def main():
     """Main entry point"""
     arg_parser = argparse.ArgumentParser()
